package markov;

import java.util.AbstractMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.Function;

public class Builder {

	private final int prefix_length;
	private final Data data;

	public Builder(int prefix_length, Data data) {
		super();
		this.prefix_length = prefix_length;
		this.data = data;
	}

	public Collection<Map.Entry<Prefix, Token>> random() {
		return produce(p -> nextRandom(p));
	}

	public Collection<Map.Entry<Prefix, Token>> average() {
		return produce(p -> data.fetch(p).average());
	}

	private Collection<Map.Entry<Prefix, Token>> produce(
			Function<Prefix, Token> producer) {
		List<Map.Entry<Prefix, Token>> result = new LinkedList<>();
		Prefix p = initPrefix();
		Token t = Token.START;
		for (int i = 0; i < 200; i++) {
			if (t == Token.END)
				break;
			t = producer.apply(p);
			result.add(new AbstractMap.SimpleEntry<Prefix, Token>(p, t));
			p = p.slide(t);
		}
		return result;
	}

	private Prefix initPrefix() {
		Token[] initToken = new Token[prefix_length];
		for (int i = 0; i < prefix_length; i++) {
			initToken[i] = Token.START;
		}
		Prefix p = new Prefix(initToken);
		return p;
	}

	private Token nextRandom(Prefix prefix) {
		return data.fetch(prefix).forRandom(nextRandomNumber());
	}

	private double nextRandomNumber() {
		return ThreadLocalRandom.current().nextDouble();
	}
}
