package markov;

import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

// FIXME rename
public class Lookup {

    public static final Lookup empty = new Lookup() {
        public Token forRandom(double random) {
            return Token.EMPTY;
        }
    };

    public Lookup() {
    }

    private final LinkedHashMap<Token, Integer> tokens = new LinkedHashMap<Token, Integer>();

    int getTotalCounts() {
        return tokens.values().stream().mapToInt(i -> i.intValue()).sum();
    }

    int getDistinctTokens() {
        return tokens.size();
    }

    int getAmount(Token t) {
        return tokens.getOrDefault(t, 0);
    }

    public void add(Token t) {
        Integer i = tokens.getOrDefault(t, 0);
        tokens.put(t, i + 1);
    }

    public Token forRandom(double random) {
        if(random <0 || random >=1) {
            throw new IllegalArgumentException("expected double [0; 1)");
        }
        int id = (int) Math.floor(random * getTotalCounts());
        int i = 0;
        for (Entry<Token, Integer> entry : tokens.entrySet()) {
            i += entry.getValue();
            if (id < i) {
                return entry.getKey();
            }
        }
        throw new IllegalStateException(String.format(
                "failed to find a random token with seed %d within %s", id,
                this));

    }

    public Token average() {
        return tokens
                .entrySet().stream().sorted((e1, e2) -> Integer
                        .compare(e2.getValue(), e1.getValue()))
                .findFirst().get().getKey();
    }

    public Set<Token> allPosible() {
        return this.tokens.keySet();
    }

    @Override
    public String toString() {
        return "Lookup [tokens= "
                + tokens.entrySet().stream()
                .sorted((e1, e2) -> Integer.compare(e2.getValue(),
                        e1.getValue()))
                .map(e -> String.format("%d*%s", e.getValue(),
                        e.getKey()))
                .collect(Collectors.joining(", "))
                + "]";
    }

}
