package markov;

public class Token {

	public static final Token START = new SpecialToken();
	public static final Token END = new SpecialToken();
	protected static final Token EMPTY = new SpecialToken();

	public Token(String content) {
	    this(content, Glyph.Type.word);
    }
	public Token(String content, Glyph.Type type) {
		super();
		this.content = content;
	}

	private String content;
	private Glyph.Type type;

	@Override
	public String toString() {
		if (this == END) {
			return "Token <END>";
		}
		if (this == START) {
			return "TOKEN <Start>";
		}
		if (this == EMPTY)
			return "TOKEN <empty>";
		return "T=[" + content + "]";
	}

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Token token = (Token) o;

        if (content != null ? !content.equals(token.content) : token.content != null) return false;
        return type == token.type;
    }

    @Override
    public int hashCode() {
        int result = content != null ? content.hashCode() : 0;
        result = 31 * result + (type != null ? type.hashCode() : 0);
        return result;
    }

    public String render(String prefix) {
		return prefix + content;
	}

	private static class SpecialToken extends Token {

		public SpecialToken() {
			super("", Glyph.Type.control);
		}

		@Override
		public String render(String prefix) {
			return this.toString();
		}

	}

}
