package markov;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertIterableEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class LookupTests {

    Lookup lookup;

    @BeforeEach
    public void initLookup() {
        lookup = new Lookup();
        lookup.add(new Token("a"));
    }


    @Test
    public void add() {
        lookup.add(new Token("b"));
        assertTrue(lookup.allPosible().contains(new Token("b")));
        assertTrue(lookup.allPosible().contains(new Token("a")));
    }

    @Test
    public void forRandom() {
        assertEquals(lookup.forRandom(0), new Token("a"));
        assertEquals(lookup.forRandom(0.5d), new Token("a"));
//        assertEquals(lookup.forRandom(1), new Token("a"));
        assertThrows(IllegalArgumentException.class, () -> lookup.forRandom(1));
        assertThrows(IllegalArgumentException.class, () -> lookup.forRandom(-1));
        assertThrows(IllegalArgumentException.class, () -> lookup.forRandom(1.001d));
        assertThrows(IllegalArgumentException.class, () -> lookup.forRandom(2));

        lookup.add(new Token("b"));
        Token a = lookup.forRandom(0);
        Token b = lookup.forRandom(0.9d);
        assertNotEquals(a, b);

        lookup.add(new Token("c"));
        a = lookup.forRandom(0);
        b = lookup.forRandom(0.51d);
        Token c = lookup.forRandom(0.9d);
        assertNotEquals(a, b);
        assertNotEquals(a, c);
        assertNotEquals(b, c);
    }

    @Test
    public void average() {
        assertEquals(lookup.average(), new Token("a"));
        lookup.add(new Token("b"));
        Token token = lookup.average();
        assertTrue(() -> token.equals(new Token("a")) || token.equals(new Token("b")));
        lookup.add(new Token("a"));
        assertEquals(lookup.average(), new Token("a"));
        lookup.add(new Token("b"));
        lookup.add(new Token("b"));
        assertEquals(lookup.average(), new Token("b"));

    }

    @Test
    public void allPossible() {
        assertIterableEquals(lookup.allPosible(),
                Arrays.asList(new Token("a")));
        lookup.add(new Token("b"));
        assertIterableEquals(lookup.allPosible(),
                Arrays.asList(new Token("a"), new Token("b")));
        lookup.add(new Token("c"));
        assertIterableEquals(lookup.allPosible(),
                Arrays.asList(new Token("a"), new Token("b"), new Token("c")));
        lookup.add(new Token("a"));
        assertIterableEquals(lookup.allPosible(),
                Arrays.asList(new Token("a"), new Token("b"), new Token("c")));
    }

    @Test
    public void getTotalCounts() {
        assertEquals(lookup.getTotalCounts(), 1);
        lookup.add(new Token("b"));
        assertEquals(lookup.getTotalCounts(), 2);
        lookup.add(new Token("b"));
        assertEquals(lookup.getTotalCounts(), 3);
    }

    @Test
    public void getDistinctTokens() {
        assertEquals(lookup.getDistinctTokens(), 1);
        lookup.add(new Token("b"));
        assertEquals(lookup.getDistinctTokens(), 2);
        lookup.add(new Token("b"));
        assertEquals(lookup.getDistinctTokens(), 2);
    }

    @Test
    public void getAmount() {
        assertEquals(1, lookup.getAmount(new Token("a")));
        lookup.add(new Token("a"));
        assertEquals(2, lookup.getAmount(new Token("a")));
        lookup.add(new Token("a"));
        assertEquals(3, lookup.getAmount(new Token("a")));
        lookup.add(new Token("b"));
    }
}